/****************************************************************************
** Meta object code from reading C++ file 'languagebuttonswidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../crow-translate/src/languagebuttonswidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'languagebuttonswidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21LanguageButtonsWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto LanguageButtonsWidget::qt_create_metaobjectdata<qt_meta_tag_ZN21LanguageButtonsWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LanguageButtonsWidget",
        "buttonChecked",
        "",
        "id",
        "languageAdded",
        "Language",
        "language",
        "languagesChanged",
        "QList<Language>",
        "languages",
        "autoLanguageChanged",
        "checkAutoButton",
        "checkButton",
        "addLanguage",
        "setAutoLanguage",
        "editLanguages",
        "savePreviousToggledButton",
        "checked",
        "checkAvailableScreenWidth",
        "minimizeWindowWidth"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'buttonChecked'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'languageAdded'
        QtMocHelpers::SignalData<void(const Language &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'languagesChanged'
        QtMocHelpers::SignalData<void(const QVector<Language> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'autoLanguageChanged'
        QtMocHelpers::SignalData<void(const Language &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'checkAutoButton'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkButton'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'addLanguage'
        QtMocHelpers::SlotData<void(const Language &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setAutoLanguage'
        QtMocHelpers::SlotData<void(const Language &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'editLanguages'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'savePreviousToggledButton'
        QtMocHelpers::SlotData<void(int, bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Bool, 17 },
        }}),
        // Slot 'checkAvailableScreenWidth'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'minimizeWindowWidth'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LanguageButtonsWidget, qt_meta_tag_ZN21LanguageButtonsWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LanguageButtonsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LanguageButtonsWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LanguageButtonsWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21LanguageButtonsWidgetE_t>.metaTypes,
    nullptr
} };

void LanguageButtonsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LanguageButtonsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->buttonChecked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->languageAdded((*reinterpret_cast<std::add_pointer_t<Language>>(_a[1]))); break;
        case 2: _t->languagesChanged((*reinterpret_cast<std::add_pointer_t<QList<Language>>>(_a[1]))); break;
        case 3: _t->autoLanguageChanged((*reinterpret_cast<std::add_pointer_t<Language>>(_a[1]))); break;
        case 4: _t->checkAutoButton(); break;
        case 5: _t->checkButton((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->addLanguage((*reinterpret_cast<std::add_pointer_t<Language>>(_a[1]))); break;
        case 7: _t->setAutoLanguage((*reinterpret_cast<std::add_pointer_t<Language>>(_a[1]))); break;
        case 8: _t->editLanguages(); break;
        case 9: _t->savePreviousToggledButton((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->checkAvailableScreenWidth(); break;
        case 11: _t->minimizeWindowWidth(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Language >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Language> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Language >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Language >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Language >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LanguageButtonsWidget::*)(int )>(_a, &LanguageButtonsWidget::buttonChecked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LanguageButtonsWidget::*)(const Language & )>(_a, &LanguageButtonsWidget::languageAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LanguageButtonsWidget::*)(const QVector<Language> & )>(_a, &LanguageButtonsWidget::languagesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LanguageButtonsWidget::*)(const Language & )>(_a, &LanguageButtonsWidget::autoLanguageChanged, 3))
            return;
    }
}

const QMetaObject *LanguageButtonsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LanguageButtonsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LanguageButtonsWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int LanguageButtonsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void LanguageButtonsWidget::buttonChecked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LanguageButtonsWidget::languageAdded(const Language & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void LanguageButtonsWidget::languagesChanged(const QVector<Language> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LanguageButtonsWidget::autoLanguageChanged(const Language & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
