/********************************************************************************
** Form generated from reading UI file 'languagesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LANGUAGESDIALOG_H
#define UI_LANGUAGESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_LanguagesDialog
{
public:
    QGridLayout *gridLayout;
    QToolButton *moveRightButton;
    QToolButton *moveDownButton;
    QListWidget *currentLanguagesListWidget;
    QToolButton *moveLeftButton;
    QLineEdit *searchEdit;
    QToolButton *moveUpButton;
    QListWidget *availableLanguagesListWidget;
    QLabel *availableLanguagesLabel;
    QLabel *currentLanguagesLabel;
    QDialogButtonBox *dialogButtonBox;

    void setupUi(QDialog *LanguagesDialog)
    {
        if (LanguagesDialog->objectName().isEmpty())
            LanguagesDialog->setObjectName("LanguagesDialog");
        LanguagesDialog->resize(450, 400);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("preferences-desktop-locale");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        LanguagesDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(LanguagesDialog);
        gridLayout->setObjectName("gridLayout");
        moveRightButton = new QToolButton(LanguagesDialog);
        moveRightButton->setObjectName("moveRightButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("arrow-right");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        moveRightButton->setIcon(icon1);
#if QT_CONFIG(shortcut)
        moveRightButton->setShortcut(QString::fromUtf8("Return"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(moveRightButton, 2, 1, 1, 1, Qt::AlignBottom);

        moveDownButton = new QToolButton(LanguagesDialog);
        moveDownButton->setObjectName("moveDownButton");
        moveDownButton->setEnabled(false);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("arrow-down");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        moveDownButton->setIcon(icon2);

        gridLayout->addWidget(moveDownButton, 5, 1, 1, 1, Qt::AlignTop);

        currentLanguagesListWidget = new QListWidget(LanguagesDialog);
        currentLanguagesListWidget->setObjectName("currentLanguagesListWidget");

        gridLayout->addWidget(currentLanguagesListWidget, 1, 2, 5, 1);

        moveLeftButton = new QToolButton(LanguagesDialog);
        moveLeftButton->setObjectName("moveLeftButton");
        moveLeftButton->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("arrow-left");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        moveLeftButton->setIcon(icon3);

        gridLayout->addWidget(moveLeftButton, 3, 1, 1, 1);

        searchEdit = new QLineEdit(LanguagesDialog);
        searchEdit->setObjectName("searchEdit");
        searchEdit->setClearButtonEnabled(true);

        gridLayout->addWidget(searchEdit, 1, 0, 1, 1);

        moveUpButton = new QToolButton(LanguagesDialog);
        moveUpButton->setObjectName("moveUpButton");
        moveUpButton->setEnabled(false);
        QIcon icon4;
        iconThemeName = QString::fromUtf8("arrow-up");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        moveUpButton->setIcon(icon4);

        gridLayout->addWidget(moveUpButton, 4, 1, 1, 1);

        availableLanguagesListWidget = new QListWidget(LanguagesDialog);
        availableLanguagesListWidget->setObjectName("availableLanguagesListWidget");
        availableLanguagesListWidget->setSortingEnabled(true);

        gridLayout->addWidget(availableLanguagesListWidget, 2, 0, 4, 1);

        availableLanguagesLabel = new QLabel(LanguagesDialog);
        availableLanguagesLabel->setObjectName("availableLanguagesLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(availableLanguagesLabel->sizePolicy().hasHeightForWidth());
        availableLanguagesLabel->setSizePolicy(sizePolicy);

        gridLayout->addWidget(availableLanguagesLabel, 0, 0, 1, 1);

        currentLanguagesLabel = new QLabel(LanguagesDialog);
        currentLanguagesLabel->setObjectName("currentLanguagesLabel");
        sizePolicy.setHeightForWidth(currentLanguagesLabel->sizePolicy().hasHeightForWidth());
        currentLanguagesLabel->setSizePolicy(sizePolicy);

        gridLayout->addWidget(currentLanguagesLabel, 0, 2, 1, 1);

        dialogButtonBox = new QDialogButtonBox(LanguagesDialog);
        dialogButtonBox->setObjectName("dialogButtonBox");
        dialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(dialogButtonBox, 6, 0, 1, 3);

        QWidget::setTabOrder(searchEdit, moveRightButton);
        QWidget::setTabOrder(moveRightButton, moveLeftButton);
        QWidget::setTabOrder(moveLeftButton, moveUpButton);
        QWidget::setTabOrder(moveUpButton, moveDownButton);
        QWidget::setTabOrder(moveDownButton, availableLanguagesListWidget);
        QWidget::setTabOrder(availableLanguagesListWidget, currentLanguagesListWidget);

        retranslateUi(LanguagesDialog);
        QObject::connect(dialogButtonBox, &QDialogButtonBox::accepted, LanguagesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(dialogButtonBox, &QDialogButtonBox::rejected, LanguagesDialog, qOverload<>(&QDialog::reject));
        QObject::connect(availableLanguagesListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem*)), LanguagesDialog, SLOT(moveLanguageRight()));
        QObject::connect(searchEdit, SIGNAL(textChanged(QString)), LanguagesDialog, SLOT(filterLanguages(QString)));
        QObject::connect(moveDownButton, SIGNAL(clicked()), LanguagesDialog, SLOT(moveLanguageDown()));
        QObject::connect(moveLeftButton, SIGNAL(clicked()), LanguagesDialog, SLOT(moveLanguageLeft()));
        QObject::connect(moveRightButton, SIGNAL(clicked()), LanguagesDialog, SLOT(moveLanguageRight()));
        QObject::connect(moveUpButton, SIGNAL(clicked()), LanguagesDialog, SLOT(moveLanguageUp()));
        QObject::connect(currentLanguagesListWidget, SIGNAL(currentRowChanged(int)), LanguagesDialog, SLOT(checkVerticalMovement(int)));

        QMetaObject::connectSlotsByName(LanguagesDialog);
    } // setupUi

    void retranslateUi(QDialog *LanguagesDialog)
    {
        LanguagesDialog->setWindowTitle(QCoreApplication::translate("LanguagesDialog", "Edit languages", nullptr));
        availableLanguagesLabel->setText(QCoreApplication::translate("LanguagesDialog", "Available languages:", nullptr));
        currentLanguagesLabel->setText(QCoreApplication::translate("LanguagesDialog", "Current languages:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LanguagesDialog: public Ui_LanguagesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LANGUAGESDIALOG_H
