// This file is autogenerated. Changes will be overwritten.
#include "UVLADIE3JM/moc_cli.cpp"
#include "UVLADIE3JM/moc_contextmenu.cpp"
#include "UVLADIE3JM/moc_instancepinger.cpp"
#include "UVLADIE3JM/moc_instancepingerdialog.cpp"
#include "UVLADIE3JM/moc_languagebuttonswidget.cpp"
#include "UVLADIE3JM/moc_languagesdialog.cpp"
#include "UVLADIE3JM/moc_mainwindow.cpp"
#include "KRARI3ZHH6/moc_ocr.cpp"
#include "32UE3KH44O/moc_abstractscreengrabber.cpp"
#include "32UE3KH44O/moc_dbusscreengrabber.cpp"
#include "32UE3KH44O/moc_genericscreengrabber.cpp"
#include "32UE3KH44O/moc_waylandgnomescreengrabber.cpp"
#include "32UE3KH44O/moc_waylandplasmascreengrabber.cpp"
#include "32UE3KH44O/moc_waylandportalscreengrabber.cpp"
#include "KRARI3ZHH6/moc_snippingarea.cpp"
#include "UVLADIE3JM/moc_onlinetranslator.cpp"
#include "UVLADIE3JM/moc_playlistplayer.cpp"
#include "UVLADIE3JM/moc_popupwindow.cpp"
#include "UVLADIE3JM/moc_provideroptionsmanager.cpp"
#include "UVLADIE3JM/moc_screenwatcher.cpp"
#include "UVLADIE3JM/moc_selection.cpp"
#include "ICMTUFM33F/moc_appsettings.cpp"
#include "IT33GJLRS4/moc_abstractautostartmanager.cpp"
#include "IT33GJLRS4/moc_portalautostartmanager.cpp"
#include "IT33GJLRS4/moc_unixautostartmanager.cpp"
#include "ICMTUFM33F/moc_settingsdialog.cpp"
#include "FVMR5IHQHM/moc_shortcutsmodel.cpp"
#include "FVMR5IHQHM/moc_shortcutsview.cpp"
#include "ICMTUFM33F/moc_tesseractparameterstablewidget.cpp"
#include "UVLADIE3JM/moc_sourcetextedit.cpp"
#include "UVLADIE3JM/moc_translationedit.cpp"
#include "JGTHPUFZYM/moc_atranslationprovider.cpp"
#include "JGTHPUFZYM/moc_copytranslationprovider.cpp"
#include "JGTHPUFZYM/moc_mozhitranslationprovider.cpp"
#include "UVLADIE3JM/moc_trayicon.cpp"
#include "ENTSRZY5KX/moc_attsprovider.cpp"
#include "ENTSRZY5KX/moc_mozhittsprovider.cpp"
#include "ENTSRZY5KX/moc_noopttsprovider.cpp"
#include "ENTSRZY5KX/moc_piperttsprovider.cpp"
#include "ENTSRZY5KX/moc_qtttsprovider.cpp"
#include "UVLADIE3JM/moc_waylandhelper.cpp"
