/****************************************************************************
** Meta object code from reading C++ file 'settingsdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../crow-translate/src/settings/settingsdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settingsdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14SettingsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto SettingsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN14SettingsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SettingsDialog",
        "translationBackendChanged",
        "",
        "ATranslationProvider::ProviderBackend",
        "newBackend",
        "ttsBackendChanged",
        "ATTSProvider::ProviderBackend",
        "piperVoicesPathChanged",
        "newPath",
        "mozhiInstanceChanged",
        "newInstance",
        "accept",
        "setCurrentPage",
        "index",
        "onProxyTypeChanged",
        "type",
        "onWindowModeChanged",
        "mode",
        "onTrayIconTypeChanged",
        "selectCustomTrayIcon",
        "setCustomTrayIconPreview",
        "iconPath",
        "detectFastestInstance",
        "selectOcrLanguagesPath",
        "onOcrLanguagesPathChanged",
        "path",
        "setupPiperVoicesPathUI",
        "selectPiperVoicesPath",
        "onPiperVoicesPathChanged",
        "onTesseractParametersCurrentItemChanged",
        "loadShortcut",
        "ShortcutItem*",
        "item",
        "updateAcceptButton",
        "acceptCurrentShortcut",
        "clearCurrentShortcut",
        "resetCurrentShortcut",
        "resetAllShortcuts",
        "restoreDefaults"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'translationBackendChanged'
        QtMocHelpers::SignalData<void(ATranslationProvider::ProviderBackend)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'ttsBackendChanged'
        QtMocHelpers::SignalData<void(ATTSProvider::ProviderBackend)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Signal 'piperVoicesPathChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'mozhiInstanceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentPage'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'onProxyTypeChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'onWindowModeChanged'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'onTrayIconTypeChanged'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'selectCustomTrayIcon'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCustomTrayIconPreview'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'detectFastestInstance'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectOcrLanguagesPath'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onOcrLanguagesPathChanged'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'setupPiperVoicesPathUI'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectPiperVoicesPath'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPiperVoicesPathChanged'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'onTesseractParametersCurrentItemChanged'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadShortcut'
        QtMocHelpers::SlotData<void(ShortcutItem *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'updateAcceptButton'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'acceptCurrentShortcut'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearCurrentShortcut'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetCurrentShortcut'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetAllShortcuts'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'restoreDefaults'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SettingsDialog, qt_meta_tag_ZN14SettingsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SettingsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SettingsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SettingsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SettingsDialogE_t>.metaTypes,
    nullptr
} };

void SettingsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SettingsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->translationBackendChanged((*reinterpret_cast<std::add_pointer_t<ATranslationProvider::ProviderBackend>>(_a[1]))); break;
        case 1: _t->ttsBackendChanged((*reinterpret_cast<std::add_pointer_t<ATTSProvider::ProviderBackend>>(_a[1]))); break;
        case 2: _t->piperVoicesPathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->mozhiInstanceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->accept(); break;
        case 5: _t->setCurrentPage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->onProxyTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->onWindowModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->onTrayIconTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->selectCustomTrayIcon(); break;
        case 10: _t->setCustomTrayIconPreview((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->detectFastestInstance(); break;
        case 12: _t->selectOcrLanguagesPath(); break;
        case 13: _t->onOcrLanguagesPathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->setupPiperVoicesPathUI(); break;
        case 15: _t->selectPiperVoicesPath(); break;
        case 16: _t->onPiperVoicesPathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->onTesseractParametersCurrentItemChanged(); break;
        case 18: _t->loadShortcut((*reinterpret_cast<std::add_pointer_t<ShortcutItem*>>(_a[1]))); break;
        case 19: _t->updateAcceptButton(); break;
        case 20: _t->acceptCurrentShortcut(); break;
        case 21: _t->clearCurrentShortcut(); break;
        case 22: _t->resetCurrentShortcut(); break;
        case 23: _t->resetAllShortcuts(); break;
        case 24: _t->restoreDefaults(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SettingsDialog::*)(ATranslationProvider::ProviderBackend )>(_a, &SettingsDialog::translationBackendChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsDialog::*)(ATTSProvider::ProviderBackend )>(_a, &SettingsDialog::ttsBackendChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsDialog::*)(const QString & )>(_a, &SettingsDialog::piperVoicesPathChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsDialog::*)(const QString & )>(_a, &SettingsDialog::mozhiInstanceChanged, 3))
            return;
    }
}

const QMetaObject *SettingsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SettingsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SettingsDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SettingsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void SettingsDialog::translationBackendChanged(ATranslationProvider::ProviderBackend _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SettingsDialog::ttsBackendChanged(ATTSProvider::ProviderBackend _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SettingsDialog::piperVoicesPathChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SettingsDialog::mozhiInstanceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
