/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "languagebuttonswidget.h"
#include "sourcetextedit.h"
#include "translationedit.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QWidget *centralwidget;
    QHBoxLayout *centralLayout;
    QGridLayout *sourceLayout;
    QToolButton *translateButton;
    QToolButton *swapButton;
    SourceTextEdit *sourceEdit;
    LanguageButtonsWidget *sourceLanguagesWidget;
    QToolButton *abortButton;
    QHBoxLayout *sourceButtonsLayout;
    QToolButton *sourcePlayPauseButton;
    QComboBox *sourceVoiceComboBox;
    QComboBox *sourceSpeakerComboBox;
    QToolButton *sourceStopButton;
    QToolButton *copySourceButton;
    QToolButton *delayedRecognizeScreenAreaButton;
    QCheckBox *autoTranslateCheckBox;
    QToolButton *clearButton;
    QVBoxLayout *translationLayout;
    LanguageButtonsWidget *translationLanguagesWidget;
    TranslationEdit *translationEdit;
    QHBoxLayout *translationButtonsLayout;
    QToolButton *translationPlayPauseButton;
    QComboBox *translationVoiceComboBox;
    QComboBox *translationSpeakerComboBox;
    QToolButton *translationStopButton;
    QSpacerItem *translationButtonsSpacer;
    QComboBox *engineComboBox;
    QToolButton *copyTranslationButton;
    QToolButton *copyAllTranslationButton;
    QToolButton *delayedTranslateScreenAreaButton;
    QToolButton *settingsButton;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(727, 246);
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        centralLayout = new QHBoxLayout(centralwidget);
        centralLayout->setObjectName("centralLayout");
        sourceLayout = new QGridLayout();
        sourceLayout->setObjectName("sourceLayout");
        translateButton = new QToolButton(centralwidget);
        translateButton->setObjectName("translateButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(translateButton->sizePolicy().hasHeightForWidth());
        translateButton->setSizePolicy(sizePolicy);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("go-next")));
        translateButton->setIcon(icon);

        sourceLayout->addWidget(translateButton, 3, 1, 1, 1);

        swapButton = new QToolButton(centralwidget);
        swapButton->setObjectName("swapButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("object-flip-horizontal")));
        swapButton->setIcon(icon1);

        sourceLayout->addWidget(swapButton, 1, 1, 1, 1);

        sourceEdit = new SourceTextEdit(centralwidget);
        sourceEdit->setObjectName("sourceEdit");
        sourceEdit->setTabChangesFocus(true);

        sourceLayout->addWidget(sourceEdit, 1, 0, 4, 1);

        sourceLanguagesWidget = new LanguageButtonsWidget(centralwidget);
        sourceLanguagesWidget->setObjectName("sourceLanguagesWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(sourceLanguagesWidget->sizePolicy().hasHeightForWidth());
        sourceLanguagesWidget->setSizePolicy(sizePolicy1);

        sourceLayout->addWidget(sourceLanguagesWidget, 0, 0, 1, 1);

        abortButton = new QToolButton(centralwidget);
        abortButton->setObjectName("abortButton");
        abortButton->setEnabled(false);
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        abortButton->setIcon(icon2);

        sourceLayout->addWidget(abortButton, 4, 1, 1, 1);

        sourceButtonsLayout = new QHBoxLayout();
        sourceButtonsLayout->setObjectName("sourceButtonsLayout");
        sourcePlayPauseButton = new QToolButton(centralwidget);
        sourcePlayPauseButton->setObjectName("sourcePlayPauseButton");
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("media-playback-start")));
        sourcePlayPauseButton->setIcon(icon3);

        sourceButtonsLayout->addWidget(sourcePlayPauseButton);

        sourceVoiceComboBox = new QComboBox(centralwidget);
        sourceVoiceComboBox->setObjectName("sourceVoiceComboBox");
        sourceVoiceComboBox->setMinimumSize(QSize(150, 0));
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(sourceVoiceComboBox->sizePolicy().hasHeightForWidth());
        sourceVoiceComboBox->setSizePolicy(sizePolicy2);
        sourceVoiceComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        sourceButtonsLayout->addWidget(sourceVoiceComboBox);

        sourceSpeakerComboBox = new QComboBox(centralwidget);
        sourceSpeakerComboBox->setObjectName("sourceSpeakerComboBox");
        sourceSpeakerComboBox->setMinimumSize(QSize(100, 0));
        sizePolicy2.setHeightForWidth(sourceSpeakerComboBox->sizePolicy().hasHeightForWidth());
        sourceSpeakerComboBox->setSizePolicy(sizePolicy2);
        sourceSpeakerComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        sourceButtonsLayout->addWidget(sourceSpeakerComboBox);

        sourceStopButton = new QToolButton(centralwidget);
        sourceStopButton->setObjectName("sourceStopButton");
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("media-playback-stop")));
        sourceStopButton->setIcon(icon4);

        sourceButtonsLayout->addWidget(sourceStopButton);

        copySourceButton = new QToolButton(centralwidget);
        copySourceButton->setObjectName("copySourceButton");
        QIcon icon5(QIcon::fromTheme(QString::fromUtf8("edit-copy")));
        copySourceButton->setIcon(icon5);

        sourceButtonsLayout->addWidget(copySourceButton);

        delayedRecognizeScreenAreaButton = new QToolButton(centralwidget);
        delayedRecognizeScreenAreaButton->setObjectName("delayedRecognizeScreenAreaButton");
        QIcon icon6(QIcon::fromTheme(QString::fromUtf8("transform-crop")));
        delayedRecognizeScreenAreaButton->setIcon(icon6);

        sourceButtonsLayout->addWidget(delayedRecognizeScreenAreaButton);

        autoTranslateCheckBox = new QCheckBox(centralwidget);
        autoTranslateCheckBox->setObjectName("autoTranslateCheckBox");
        sizePolicy2.setHeightForWidth(autoTranslateCheckBox->sizePolicy().hasHeightForWidth());
        autoTranslateCheckBox->setSizePolicy(sizePolicy2);

        sourceButtonsLayout->addWidget(autoTranslateCheckBox);


        sourceLayout->addLayout(sourceButtonsLayout, 5, 0, 1, 1);

        clearButton = new QToolButton(centralwidget);
        clearButton->setObjectName("clearButton");
        QIcon icon7(QIcon::fromTheme(QString::fromUtf8("edit-clear-all")));
        clearButton->setIcon(icon7);

        sourceLayout->addWidget(clearButton, 2, 1, 1, 1);


        centralLayout->addLayout(sourceLayout);

        translationLayout = new QVBoxLayout();
        translationLayout->setObjectName("translationLayout");
        translationLanguagesWidget = new LanguageButtonsWidget(centralwidget);
        translationLanguagesWidget->setObjectName("translationLanguagesWidget");
        translationLanguagesWidget->setLayoutDirection(Qt::LayoutDirection::RightToLeft);

        translationLayout->addWidget(translationLanguagesWidget);

        translationEdit = new TranslationEdit(centralwidget);
        translationEdit->setObjectName("translationEdit");
        translationEdit->setStyleSheet(QString::fromUtf8(""));
        translationEdit->setReadOnly(true);

        translationLayout->addWidget(translationEdit);

        translationButtonsLayout = new QHBoxLayout();
        translationButtonsLayout->setObjectName("translationButtonsLayout");
        translationPlayPauseButton = new QToolButton(centralwidget);
        translationPlayPauseButton->setObjectName("translationPlayPauseButton");
        translationPlayPauseButton->setIcon(icon3);

        translationButtonsLayout->addWidget(translationPlayPauseButton);

        translationVoiceComboBox = new QComboBox(centralwidget);
        translationVoiceComboBox->setObjectName("translationVoiceComboBox");
        translationVoiceComboBox->setMinimumSize(QSize(150, 0));
        sizePolicy2.setHeightForWidth(translationVoiceComboBox->sizePolicy().hasHeightForWidth());
        translationVoiceComboBox->setSizePolicy(sizePolicy2);
        translationVoiceComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        translationButtonsLayout->addWidget(translationVoiceComboBox);

        translationSpeakerComboBox = new QComboBox(centralwidget);
        translationSpeakerComboBox->setObjectName("translationSpeakerComboBox");
        translationSpeakerComboBox->setMinimumSize(QSize(100, 0));
        sizePolicy2.setHeightForWidth(translationSpeakerComboBox->sizePolicy().hasHeightForWidth());
        translationSpeakerComboBox->setSizePolicy(sizePolicy2);
        translationSpeakerComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        translationButtonsLayout->addWidget(translationSpeakerComboBox);

        translationStopButton = new QToolButton(centralwidget);
        translationStopButton->setObjectName("translationStopButton");
        translationStopButton->setIcon(icon4);

        translationButtonsLayout->addWidget(translationStopButton);

        translationButtonsSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        translationButtonsLayout->addItem(translationButtonsSpacer);

        engineComboBox = new QComboBox(centralwidget);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/engines/google.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon8, QString::fromUtf8("Google"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/engines/yandex.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon9, QString::fromUtf8("Yandex"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/engines/deepl.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon10, QString::fromUtf8("DeepL"));
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/icons/engines/duckduckgo.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon11, QString::fromUtf8("DuckDuckGo"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/icons/engines/libretranslate.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon12, QString::fromUtf8("LibreTranslate"));
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/icons/engines/mymemory.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon13, QString::fromUtf8("MyMemory"));
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/icons/engines/reverso.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon14, QString::fromUtf8("Reverso"));
        engineComboBox->setObjectName("engineComboBox");

        translationButtonsLayout->addWidget(engineComboBox);

        copyTranslationButton = new QToolButton(centralwidget);
        copyTranslationButton->setObjectName("copyTranslationButton");
        copyTranslationButton->setIcon(icon5);

        translationButtonsLayout->addWidget(copyTranslationButton);

        copyAllTranslationButton = new QToolButton(centralwidget);
        copyAllTranslationButton->setObjectName("copyAllTranslationButton");
        QIcon icon15(QIcon::fromTheme(QString::fromUtf8("edit-select-all")));
        copyAllTranslationButton->setIcon(icon15);

        translationButtonsLayout->addWidget(copyAllTranslationButton);

        delayedTranslateScreenAreaButton = new QToolButton(centralwidget);
        delayedTranslateScreenAreaButton->setObjectName("delayedTranslateScreenAreaButton");
        delayedTranslateScreenAreaButton->setIcon(icon6);

        translationButtonsLayout->addWidget(delayedTranslateScreenAreaButton);

        settingsButton = new QToolButton(centralwidget);
        settingsButton->setObjectName("settingsButton");
        QIcon icon16(QIcon::fromTheme(QString::fromUtf8("preferences-other")));
        settingsButton->setIcon(icon16);

        translationButtonsLayout->addWidget(settingsButton);


        translationLayout->addLayout(translationButtonsLayout);


        centralLayout->addLayout(translationLayout);

        MainWindow->setCentralWidget(centralwidget);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Crow Translate", nullptr));
#if QT_CONFIG(tooltip)
        translateButton->setToolTip(QCoreApplication::translate("MainWindow", "Translate", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        swapButton->setToolTip(QCoreApplication::translate("MainWindow", "Swap languages", nullptr));
#endif // QT_CONFIG(tooltip)
        sourceEdit->setPlaceholderText(QCoreApplication::translate("MainWindow", "Source", nullptr));
#if QT_CONFIG(tooltip)
        abortButton->setToolTip(QCoreApplication::translate("MainWindow", "Cancel", nullptr));
#endif // QT_CONFIG(tooltip)
        sourcePlayPauseButton->setText(QString());
        sourceStopButton->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
#if QT_CONFIG(tooltip)
        copySourceButton->setToolTip(QCoreApplication::translate("MainWindow", "Copy source text to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        delayedRecognizeScreenAreaButton->setToolTip(QCoreApplication::translate("MainWindow", "Recognize screen area with delay", nullptr));
#endif // QT_CONFIG(tooltip)
        autoTranslateCheckBox->setText(QCoreApplication::translate("MainWindow", "Auto-translation", nullptr));
#if QT_CONFIG(tooltip)
        clearButton->setToolTip(QCoreApplication::translate("MainWindow", "Clear", nullptr));
#endif // QT_CONFIG(tooltip)
        translationEdit->setPlaceholderText(QCoreApplication::translate("MainWindow", "Translation", nullptr));
        translationPlayPauseButton->setText(QString());
        translationStopButton->setText(QCoreApplication::translate("MainWindow", "...", nullptr));

#if QT_CONFIG(tooltip)
        copyTranslationButton->setToolTip(QCoreApplication::translate("MainWindow", "Copy translation to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        copyAllTranslationButton->setToolTip(QCoreApplication::translate("MainWindow", "Copy all translation data to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        delayedTranslateScreenAreaButton->setToolTip(QCoreApplication::translate("MainWindow", "Translate screen area with delay", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        settingsButton->setToolTip(QCoreApplication::translate("MainWindow", "Application settings", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
