/****************************************************************************
** Meta object code from reading C++ file 'onlinetranslator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../crow-translate/src/onlinetranslator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'onlinetranslator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16OnlineTranslatorE_t {};
} // unnamed namespace

template <> constexpr inline auto OnlineTranslator::qt_create_metaobjectdata<qt_meta_tag_ZN16OnlineTranslatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OnlineTranslator",
        "finished",
        "",
        "skipGarbageText",
        "requestTranslate",
        "parseTranslate",
        "Language",
        "NoLanguage",
        "Auto",
        "Afrikaans",
        "Albanian",
        "Amharic",
        "Arabic",
        "Armenian",
        "Assamese",
        "Aymara",
        "Azerbaijani",
        "Bajan",
        "BalkanGipsy",
        "Bambara",
        "Bangla",
        "Bashkir",
        "Basque",
        "Belarusian",
        "Bemba",
        "Bhojpuri",
        "Bislama",
        "Bosnian",
        "Breton",
        "Bulgarian",
        "Cantonese",
        "Catalan",
        "Cebuano",
        "Chamorro",
        "Chichewa",
        "ChineseLiterary",
        "ChineseSimplified",
        "ChineseTraditional",
        "Chuvash",
        "Comorian",
        "Coptic",
        "Corsican",
        "AntiguanCreole",
        "BahamianCreole",
        "GrenadianCreole",
        "GuyaneseCreole",
        "JamaicanCreole",
        "VincentianCreole",
        "VirginIslandsCreole",
        "SaintLucianCreole",
        "SeselwaCreole",
        "UpperGuineaCreole",
        "Croatian",
        "Czech",
        "Danish",
        "Dari",
        "Divehi",
        "Dogri",
        "Dutch",
        "Dzongkha",
        "Elvish",
        "Emoji",
        "English",
        "Esperanto",
        "Estonian",
        "Ewe",
        "Fanagalo",
        "Faroese",
        "Fijian",
        "Filipino",
        "Finnish",
        "French",
        "FrenchCanada",
        "Frisian",
        "Galician",
        "Ganda",
        "Georgian",
        "German",
        "Greek",
        "GreekClassical",
        "Guarani",
        "Gujarati",
        "HaitianCreole",
        "Hausa",
        "Hawaiian",
        "Hebrew",
        "HillMari",
        "Hindi",
        "Hmong",
        "HmongDaw",
        "Hungarian",
        "Icelandic",
        "Igbo",
        "Ilocano",
        "Indonesian",
        "Inuinnaqtun",
        "Inuktitut",
        "InuktitutGreenlandic",
        "InuktitutLatin",
        "Irish",
        "Italian",
        "Japanese",
        "Javanese",
        "Kabuverdianu",
        "Kabylian",
        "Kannada",
        "Kazakh",
        "KazakhLatin",
        "Khmer",
        "Kinyarwanda",
        "Kirundi",
        "Klingon",
        "Konkani",
        "Korean",
        "Krio",
        "KurdishCentral",
        "KurdishNorthern",
        "KurdishSorani",
        "Kyrgyz",
        "Lao",
        "Latin",
        "Latvian",
        "Lingala",
        "Lithuanian",
        "LowerSorbian",
        "Luganda",
        "Luxembourgish",
        "Macedonian",
        "Maithili",
        "Malagasy",
        "Malay",
        "Malayalam",
        "Maltese",
        "ManxGaelic",
        "Marathi",
        "Mari",
        "Marshallese",
        "Meiteilon",
        "Mende",
        "Mizo",
        "Mongolian",
        "MongolianCyrillic",
        "MongolianTraditional",
        "Morisyen",
        "Myanmar",
        "Maori",
        "Nepali",
        "Niuean",
        "Norwegian",
        "Nyanja",
        "Odia",
        "Oromo",
        "Palauan",
        "Papiamentu",
        "Pashto",
        "Persian",
        "Pijin",
        "Polish",
        "PortugueseBrazilian",
        "PortuguesePortugal",
        "Potawatomi",
        "Punjabi",
        "Quechua",
        "QueretaroOtomi",
        "Romanian",
        "Rundi",
        "Russian",
        "Samoan",
        "Sango",
        "Sanskrit",
        "ScotsGaelic",
        "SerbianCyrillic",
        "SerbianLatin",
        "Sesotho",
        "SesothoSaLeboa",
        "Setswana",
        "Shona",
        "Sindhi",
        "Sinhala",
        "Slovak",
        "Slovenian",
        "Somali",
        "Spanish",
        "SrananTongo",
        "Sundanese",
        "Swahili",
        "Swedish",
        "Syriac",
        "Tahitian",
        "Tajik",
        "Tamashek",
        "Tamil",
        "Tatar",
        "Telugu",
        "Tetum",
        "Thai",
        "Tibetan",
        "Tigrinya",
        "TokPisin",
        "Tokelauan",
        "Tongan",
        "Tsonga",
        "Turkish",
        "Turkmen",
        "Tuvaluan",
        "Twi",
        "Udmurt",
        "Ukrainian",
        "Uma",
        "UpperSorbian",
        "Urdu",
        "Uyghur",
        "UzbekCyrillic",
        "UzbekLatin",
        "Vietnamese",
        "Wallisian",
        "Welsh",
        "Wolof",
        "Xhosa",
        "Yakut",
        "Yiddish",
        "Yoruba",
        "YucatecMaya",
        "Zulu",
        "Engine",
        "Google",
        "Yandex",
        "Deepl",
        "Duckduckgo",
        "LibreTranslate",
        "Mymemory",
        "Reverso"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'skipGarbageText'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestTranslate'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'parseTranslate'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Language'
        QtMocHelpers::EnumData<enum Language>(6, 6, QMC::EnumFlags{}).add({
            {    7, Language::NoLanguage },
            {    8, Language::Auto },
            {    9, Language::Afrikaans },
            {   10, Language::Albanian },
            {   11, Language::Amharic },
            {   12, Language::Arabic },
            {   13, Language::Armenian },
            {   14, Language::Assamese },
            {   15, Language::Aymara },
            {   16, Language::Azerbaijani },
            {   17, Language::Bajan },
            {   18, Language::BalkanGipsy },
            {   19, Language::Bambara },
            {   20, Language::Bangla },
            {   21, Language::Bashkir },
            {   22, Language::Basque },
            {   23, Language::Belarusian },
            {   24, Language::Bemba },
            {   25, Language::Bhojpuri },
            {   26, Language::Bislama },
            {   27, Language::Bosnian },
            {   28, Language::Breton },
            {   29, Language::Bulgarian },
            {   30, Language::Cantonese },
            {   31, Language::Catalan },
            {   32, Language::Cebuano },
            {   33, Language::Chamorro },
            {   34, Language::Chichewa },
            {   35, Language::ChineseLiterary },
            {   36, Language::ChineseSimplified },
            {   37, Language::ChineseTraditional },
            {   38, Language::Chuvash },
            {   39, Language::Comorian },
            {   40, Language::Coptic },
            {   41, Language::Corsican },
            {   42, Language::AntiguanCreole },
            {   43, Language::BahamianCreole },
            {   44, Language::GrenadianCreole },
            {   45, Language::GuyaneseCreole },
            {   46, Language::JamaicanCreole },
            {   47, Language::VincentianCreole },
            {   48, Language::VirginIslandsCreole },
            {   49, Language::SaintLucianCreole },
            {   50, Language::SeselwaCreole },
            {   51, Language::UpperGuineaCreole },
            {   52, Language::Croatian },
            {   53, Language::Czech },
            {   54, Language::Danish },
            {   55, Language::Dari },
            {   56, Language::Divehi },
            {   57, Language::Dogri },
            {   58, Language::Dutch },
            {   59, Language::Dzongkha },
            {   60, Language::Elvish },
            {   61, Language::Emoji },
            {   62, Language::English },
            {   63, Language::Esperanto },
            {   64, Language::Estonian },
            {   65, Language::Ewe },
            {   66, Language::Fanagalo },
            {   67, Language::Faroese },
            {   68, Language::Fijian },
            {   69, Language::Filipino },
            {   70, Language::Finnish },
            {   71, Language::French },
            {   72, Language::FrenchCanada },
            {   73, Language::Frisian },
            {   74, Language::Galician },
            {   75, Language::Ganda },
            {   76, Language::Georgian },
            {   77, Language::German },
            {   78, Language::Greek },
            {   79, Language::GreekClassical },
            {   80, Language::Guarani },
            {   81, Language::Gujarati },
            {   82, Language::HaitianCreole },
            {   83, Language::Hausa },
            {   84, Language::Hawaiian },
            {   85, Language::Hebrew },
            {   86, Language::HillMari },
            {   87, Language::Hindi },
            {   88, Language::Hmong },
            {   89, Language::HmongDaw },
            {   90, Language::Hungarian },
            {   91, Language::Icelandic },
            {   92, Language::Igbo },
            {   93, Language::Ilocano },
            {   94, Language::Indonesian },
            {   95, Language::Inuinnaqtun },
            {   96, Language::Inuktitut },
            {   97, Language::InuktitutGreenlandic },
            {   98, Language::InuktitutLatin },
            {   99, Language::Irish },
            {  100, Language::Italian },
            {  101, Language::Japanese },
            {  102, Language::Javanese },
            {  103, Language::Kabuverdianu },
            {  104, Language::Kabylian },
            {  105, Language::Kannada },
            {  106, Language::Kazakh },
            {  107, Language::KazakhLatin },
            {  108, Language::Khmer },
            {  109, Language::Kinyarwanda },
            {  110, Language::Kirundi },
            {  111, Language::Klingon },
            {  112, Language::Konkani },
            {  113, Language::Korean },
            {  114, Language::Krio },
            {  115, Language::KurdishCentral },
            {  116, Language::KurdishNorthern },
            {  117, Language::KurdishSorani },
            {  118, Language::Kyrgyz },
            {  119, Language::Lao },
            {  120, Language::Latin },
            {  121, Language::Latvian },
            {  122, Language::Lingala },
            {  123, Language::Lithuanian },
            {  124, Language::LowerSorbian },
            {  125, Language::Luganda },
            {  126, Language::Luxembourgish },
            {  127, Language::Macedonian },
            {  128, Language::Maithili },
            {  129, Language::Malagasy },
            {  130, Language::Malay },
            {  131, Language::Malayalam },
            {  132, Language::Maltese },
            {  133, Language::ManxGaelic },
            {  134, Language::Marathi },
            {  135, Language::Mari },
            {  136, Language::Marshallese },
            {  137, Language::Meiteilon },
            {  138, Language::Mende },
            {  139, Language::Mizo },
            {  140, Language::Mongolian },
            {  141, Language::MongolianCyrillic },
            {  142, Language::MongolianTraditional },
            {  143, Language::Morisyen },
            {  144, Language::Myanmar },
            {  145, Language::Maori },
            {  146, Language::Nepali },
            {  147, Language::Niuean },
            {  148, Language::Norwegian },
            {  149, Language::Nyanja },
            {  150, Language::Odia },
            {  151, Language::Oromo },
            {  152, Language::Palauan },
            {  153, Language::Papiamentu },
            {  154, Language::Pashto },
            {  155, Language::Persian },
            {  156, Language::Pijin },
            {  157, Language::Polish },
            {  158, Language::PortugueseBrazilian },
            {  159, Language::PortuguesePortugal },
            {  160, Language::Potawatomi },
            {  161, Language::Punjabi },
            {  162, Language::Quechua },
            {  163, Language::QueretaroOtomi },
            {  164, Language::Romanian },
            {  165, Language::Rundi },
            {  166, Language::Russian },
            {  167, Language::Samoan },
            {  168, Language::Sango },
            {  169, Language::Sanskrit },
            {  170, Language::ScotsGaelic },
            {  171, Language::SerbianCyrillic },
            {  172, Language::SerbianLatin },
            {  173, Language::Sesotho },
            {  174, Language::SesothoSaLeboa },
            {  175, Language::Setswana },
            {  176, Language::Shona },
            {  177, Language::Sindhi },
            {  178, Language::Sinhala },
            {  179, Language::Slovak },
            {  180, Language::Slovenian },
            {  181, Language::Somali },
            {  182, Language::Spanish },
            {  183, Language::SrananTongo },
            {  184, Language::Sundanese },
            {  185, Language::Swahili },
            {  186, Language::Swedish },
            {  187, Language::Syriac },
            {  188, Language::Tahitian },
            {  189, Language::Tajik },
            {  190, Language::Tamashek },
            {  191, Language::Tamil },
            {  192, Language::Tatar },
            {  193, Language::Telugu },
            {  194, Language::Tetum },
            {  195, Language::Thai },
            {  196, Language::Tibetan },
            {  197, Language::Tigrinya },
            {  198, Language::TokPisin },
            {  199, Language::Tokelauan },
            {  200, Language::Tongan },
            {  201, Language::Tsonga },
            {  202, Language::Turkish },
            {  203, Language::Turkmen },
            {  204, Language::Tuvaluan },
            {  205, Language::Twi },
            {  206, Language::Udmurt },
            {  207, Language::Ukrainian },
            {  208, Language::Uma },
            {  209, Language::UpperSorbian },
            {  210, Language::Urdu },
            {  211, Language::Uyghur },
            {  212, Language::UzbekCyrillic },
            {  213, Language::UzbekLatin },
            {  214, Language::Vietnamese },
            {  215, Language::Wallisian },
            {  216, Language::Welsh },
            {  217, Language::Wolof },
            {  218, Language::Xhosa },
            {  219, Language::Yakut },
            {  220, Language::Yiddish },
            {  221, Language::Yoruba },
            {  222, Language::YucatecMaya },
            {  223, Language::Zulu },
        }),
        // enum 'Engine'
        QtMocHelpers::EnumData<enum Engine>(224, 224, QMC::EnumFlags{}).add({
            {  225, Engine::Google },
            {  226, Engine::Yandex },
            {  227, Engine::Deepl },
            {  228, Engine::Duckduckgo },
            {  229, Engine::LibreTranslate },
            {  230, Engine::Mymemory },
            {  231, Engine::Reverso },
        }),
    };
    return QtMocHelpers::metaObjectData<OnlineTranslator, qt_meta_tag_ZN16OnlineTranslatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OnlineTranslator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OnlineTranslatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OnlineTranslatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16OnlineTranslatorE_t>.metaTypes,
    nullptr
} };

void OnlineTranslator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OnlineTranslator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->skipGarbageText(); break;
        case 2: _t->requestTranslate(); break;
        case 3: _t->parseTranslate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OnlineTranslator::*)()>(_a, &OnlineTranslator::finished, 0))
            return;
    }
}

const QMetaObject *OnlineTranslator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OnlineTranslator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OnlineTranslatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OnlineTranslator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OnlineTranslator::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
