/********************************************************************************
** Form generated from reading UI file 'languagebuttonswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LANGUAGEBUTTONSWIDGET_H
#define UI_LANGUAGEBUTTONSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LanguageButtonsWidget
{
public:
    QHBoxLayout *languagesLayout;
    QToolButton *editButton;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *LanguageButtonsWidget)
    {
        if (LanguageButtonsWidget->objectName().isEmpty())
            LanguageButtonsWidget->setObjectName("LanguageButtonsWidget");
        LanguageButtonsWidget->resize(84, 32);
        languagesLayout = new QHBoxLayout(LanguageButtonsWidget);
        languagesLayout->setSpacing(0);
        languagesLayout->setObjectName("languagesLayout");
        languagesLayout->setContentsMargins(0, 0, 0, 0);
        editButton = new QToolButton(LanguageButtonsWidget);
        editButton->setObjectName("editButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-edit");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        editButton->setIcon(icon);

        languagesLayout->addWidget(editButton);

        horizontalSpacer = new QSpacerItem(52, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        languagesLayout->addItem(horizontalSpacer);


        retranslateUi(LanguageButtonsWidget);
        QObject::connect(editButton, SIGNAL(clicked()), LanguageButtonsWidget, SLOT(editLanguages()));

        QMetaObject::connectSlotsByName(LanguageButtonsWidget);
    } // setupUi

    void retranslateUi(QWidget *LanguageButtonsWidget)
    {
#if QT_CONFIG(tooltip)
        editButton->setToolTip(QCoreApplication::translate("LanguageButtonsWidget", "Edit languages", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)LanguageButtonsWidget;
    } // retranslateUi

};

namespace Ui {
    class LanguageButtonsWidget: public Ui_LanguageButtonsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LANGUAGEBUTTONSWIDGET_H
