/********************************************************************************
** Form generated from reading UI file 'popupwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_POPUPWINDOW_H
#define UI_POPUPWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "languagebuttonswidget.h"
#include "translationedit.h"

QT_BEGIN_NAMESPACE

class Ui_PopupWindow
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *topLayout;
    LanguageButtonsWidget *sourceLanguagesWidget;
    QToolButton *swapButton;
    LanguageButtonsWidget *translationLanguagesWidget;
    TranslationEdit *translationEdit;
    QHBoxLayout *buttomLayout;
    QToolButton *sourcePlayPauseButton;
    QComboBox *sourceVoiceComboBox;
    QComboBox *sourceSpeakerComboBox;
    QToolButton *sourceStopButton;
    QToolButton *copySourceButton;
    QSpacerItem *buttomSpacer;
    QComboBox *engineComboBox;
    QToolButton *translationPlayPauseButton;
    QComboBox *translationVoiceComboBox;
    QComboBox *translationSpeakerComboBox;
    QToolButton *translationStopButton;
    QToolButton *copyAllTranslationButton;
    QToolButton *copyTranslationButton;

    void setupUi(QWidget *PopupWindow)
    {
        if (PopupWindow->objectName().isEmpty())
            PopupWindow->setObjectName("PopupWindow");
        PopupWindow->resize(655, 296);
        PopupWindow->setWindowTitle(QString::fromUtf8("Crow Translate"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/app/black.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        PopupWindow->setWindowIcon(icon);
        verticalLayout_2 = new QVBoxLayout(PopupWindow);
        verticalLayout_2->setObjectName("verticalLayout_2");
        topLayout = new QHBoxLayout();
        topLayout->setSpacing(0);
        topLayout->setObjectName("topLayout");
        sourceLanguagesWidget = new LanguageButtonsWidget(PopupWindow);
        sourceLanguagesWidget->setObjectName("sourceLanguagesWidget");

        topLayout->addWidget(sourceLanguagesWidget);

        swapButton = new QToolButton(PopupWindow);
        swapButton->setObjectName("swapButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("object-flip-horizontal")));
        swapButton->setIcon(icon1);

        topLayout->addWidget(swapButton);

        translationLanguagesWidget = new LanguageButtonsWidget(PopupWindow);
        translationLanguagesWidget->setObjectName("translationLanguagesWidget");
        translationLanguagesWidget->setLayoutDirection(Qt::LayoutDirection::RightToLeft);

        topLayout->addWidget(translationLanguagesWidget);


        verticalLayout_2->addLayout(topLayout);

        translationEdit = new TranslationEdit(PopupWindow);
        translationEdit->setObjectName("translationEdit");
        translationEdit->setReadOnly(true);

        verticalLayout_2->addWidget(translationEdit);

        buttomLayout = new QHBoxLayout();
        buttomLayout->setObjectName("buttomLayout");
        sourcePlayPauseButton = new QToolButton(PopupWindow);
        sourcePlayPauseButton->setObjectName("sourcePlayPauseButton");

        buttomLayout->addWidget(sourcePlayPauseButton);

        sourceVoiceComboBox = new QComboBox(PopupWindow);
        sourceVoiceComboBox->setObjectName("sourceVoiceComboBox");
        sourceVoiceComboBox->setMinimumSize(QSize(150, 0));
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(sourceVoiceComboBox->sizePolicy().hasHeightForWidth());
        sourceVoiceComboBox->setSizePolicy(sizePolicy);
        sourceVoiceComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        buttomLayout->addWidget(sourceVoiceComboBox);

        sourceSpeakerComboBox = new QComboBox(PopupWindow);
        sourceSpeakerComboBox->setObjectName("sourceSpeakerComboBox");
        sourceSpeakerComboBox->setMinimumSize(QSize(100, 0));
        sizePolicy.setHeightForWidth(sourceSpeakerComboBox->sizePolicy().hasHeightForWidth());
        sourceSpeakerComboBox->setSizePolicy(sizePolicy);
        sourceSpeakerComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        buttomLayout->addWidget(sourceSpeakerComboBox);

        sourceStopButton = new QToolButton(PopupWindow);
        sourceStopButton->setObjectName("sourceStopButton");

        buttomLayout->addWidget(sourceStopButton);

        copySourceButton = new QToolButton(PopupWindow);
        copySourceButton->setObjectName("copySourceButton");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("edit-copy")));
        copySourceButton->setIcon(icon2);

        buttomLayout->addWidget(copySourceButton);

        buttomSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        buttomLayout->addItem(buttomSpacer);

        engineComboBox = new QComboBox(PopupWindow);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/engines/google.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon3, QString::fromUtf8("Google"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/engines/yandex.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon4, QString::fromUtf8("Yandex"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/engines/deepl.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon5, QString::fromUtf8("DeepL"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/engines/duckduckgo.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon6, QString::fromUtf8("DuckDuckGo"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/engines/libretranslate.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon7, QString::fromUtf8("LibreTranslate"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/engines/mymemory.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon8, QString::fromUtf8("MyMemory"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/engines/reverso.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        engineComboBox->addItem(icon9, QString::fromUtf8("Reverso"));
        engineComboBox->setObjectName("engineComboBox");

        buttomLayout->addWidget(engineComboBox);

        translationPlayPauseButton = new QToolButton(PopupWindow);
        translationPlayPauseButton->setObjectName("translationPlayPauseButton");

        buttomLayout->addWidget(translationPlayPauseButton);

        translationVoiceComboBox = new QComboBox(PopupWindow);
        translationVoiceComboBox->setObjectName("translationVoiceComboBox");
        translationVoiceComboBox->setMinimumSize(QSize(150, 0));
        sizePolicy.setHeightForWidth(translationVoiceComboBox->sizePolicy().hasHeightForWidth());
        translationVoiceComboBox->setSizePolicy(sizePolicy);
        translationVoiceComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        buttomLayout->addWidget(translationVoiceComboBox);

        translationSpeakerComboBox = new QComboBox(PopupWindow);
        translationSpeakerComboBox->setObjectName("translationSpeakerComboBox");
        translationSpeakerComboBox->setMinimumSize(QSize(100, 0));
        sizePolicy.setHeightForWidth(translationSpeakerComboBox->sizePolicy().hasHeightForWidth());
        translationSpeakerComboBox->setSizePolicy(sizePolicy);
        translationSpeakerComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        buttomLayout->addWidget(translationSpeakerComboBox);

        translationStopButton = new QToolButton(PopupWindow);
        translationStopButton->setObjectName("translationStopButton");

        buttomLayout->addWidget(translationStopButton);

        copyAllTranslationButton = new QToolButton(PopupWindow);
        copyAllTranslationButton->setObjectName("copyAllTranslationButton");
        QIcon icon10(QIcon::fromTheme(QString::fromUtf8("edit-select-all")));
        copyAllTranslationButton->setIcon(icon10);

        buttomLayout->addWidget(copyAllTranslationButton);

        copyTranslationButton = new QToolButton(PopupWindow);
        copyTranslationButton->setObjectName("copyTranslationButton");
        copyTranslationButton->setIcon(icon2);

        buttomLayout->addWidget(copyTranslationButton);


        verticalLayout_2->addLayout(buttomLayout);


        retranslateUi(PopupWindow);

        QMetaObject::connectSlotsByName(PopupWindow);
    } // setupUi

    void retranslateUi(QWidget *PopupWindow)
    {
#if QT_CONFIG(tooltip)
        swapButton->setToolTip(QCoreApplication::translate("PopupWindow", "Swap languages", nullptr));
#endif // QT_CONFIG(tooltip)
        sourcePlayPauseButton->setText(QCoreApplication::translate("PopupWindow", "...", nullptr));
        sourceStopButton->setText(QCoreApplication::translate("PopupWindow", "...", nullptr));
#if QT_CONFIG(tooltip)
        copySourceButton->setToolTip(QCoreApplication::translate("PopupWindow", "Copy source text to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)

        translationPlayPauseButton->setText(QCoreApplication::translate("PopupWindow", "...", nullptr));
        translationStopButton->setText(QCoreApplication::translate("PopupWindow", "...", nullptr));
#if QT_CONFIG(tooltip)
        copyAllTranslationButton->setToolTip(QCoreApplication::translate("PopupWindow", "Copy all translation data to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        copyTranslationButton->setToolTip(QCoreApplication::translate("PopupWindow", "Copy translation to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)PopupWindow;
    } // retranslateUi

};

namespace Ui {
    class PopupWindow: public Ui_PopupWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_POPUPWINDOW_H
