/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../crow-translate/src/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "D-Bus Interface",
        "org.kde.CrowTranslate.MainWindow",
        "translationRequested",
        "",
        "inputText",
        "Language",
        "translationLang",
        "sourceLang",
        "translationAccepted",
        "resetTranslator",
        "ttsStateChanged",
        "QTextToSpeech::State",
        "newState",
        "onTTSError",
        "QTextToSpeech::ErrorReason",
        "reason",
        "errorString",
        "translatorStateChanged",
        "ATranslationProvider::State",
        "open",
        "quit",
        "openSettings",
        "toggleOcrNegate",
        "on_translateButton_clicked",
        "translateSelection",
        "speakSelection",
        "speakTranslatedSelection",
        "stopSpeaking",
        "playPauseSpeaking",
        "copyTranslatedSelection",
        "recognizeScreenArea",
        "translateScreenArea",
        "delayedRecognizeScreenArea",
        "delayedTranslateScreenArea",
        "setOrientation",
        "Qt::ScreenOrientation",
        "orientation",
        "onSourceLanguageChanged",
        "id",
        "onDestinationLanguageChanged",
        "handleAutoTranslation",
        "updateVoiceComboBoxes",
        "updateSpeakerComboBoxes",
        "on_swapButton_clicked",
        "on_abortButton_clicked",
        "on_copySourceButton_clicked",
        "on_delayedRecognizeScreenAreaButton_clicked",
        "on_clearButton_clicked",
        "on_copyTranslationButton_clicked",
        "on_copyAllTranslationButton_clicked",
        "on_delayedTranslateScreenAreaButton_clicked",
        "clearText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'translationRequested'
        QtMocHelpers::SignalData<void(const QString &, const Language &, const Language &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 },
        }}),
        // Signal 'translationAccepted'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetTranslator'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ttsStateChanged'
        QtMocHelpers::SlotData<void(QTextToSpeech::State)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onTTSError'
        QtMocHelpers::SlotData<void(QTextToSpeech::ErrorReason, const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'translatorStateChanged'
        QtMocHelpers::SlotData<void(ATranslationProvider::State)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 13 },
        }}),
        // Slot 'open'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'openSettings'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'toggleOcrNegate'
        QtMocHelpers::SlotData<void()>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'on_translateButton_clicked'
        QtMocHelpers::SlotData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'translateSelection'
        QtMocHelpers::SlotData<void()>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'speakSelection'
        QtMocHelpers::SlotData<void()>(26, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'speakTranslatedSelection'
        QtMocHelpers::SlotData<void()>(27, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'stopSpeaking'
        QtMocHelpers::SlotData<void()>(28, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'playPauseSpeaking'
        QtMocHelpers::SlotData<void()>(29, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'copyTranslatedSelection'
        QtMocHelpers::SlotData<void()>(30, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'recognizeScreenArea'
        QtMocHelpers::SlotData<void()>(31, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'translateScreenArea'
        QtMocHelpers::SlotData<void()>(32, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'delayedRecognizeScreenArea'
        QtMocHelpers::SlotData<void()>(33, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'delayedTranslateScreenArea'
        QtMocHelpers::SlotData<void()>(34, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'setOrientation'
        QtMocHelpers::SlotData<void(Qt::ScreenOrientation)>(35, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'onSourceLanguageChanged'
        QtMocHelpers::SlotData<void(int)>(38, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'onDestinationLanguageChanged'
        QtMocHelpers::SlotData<void(int)>(40, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'handleAutoTranslation'
        QtMocHelpers::SlotData<void()>(41, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateVoiceComboBoxes'
        QtMocHelpers::SlotData<void()>(42, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSpeakerComboBoxes'
        QtMocHelpers::SlotData<void()>(43, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_swapButton_clicked'
        QtMocHelpers::SlotData<void()>(44, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_abortButton_clicked'
        QtMocHelpers::SlotData<void()>(45, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_copySourceButton_clicked'
        QtMocHelpers::SlotData<void()>(46, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_delayedRecognizeScreenAreaButton_clicked'
        QtMocHelpers::SlotData<void()>(47, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_clearButton_clicked'
        QtMocHelpers::SlotData<void()>(48, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_copyTranslationButton_clicked'
        QtMocHelpers::SlotData<void()>(49, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_copyAllTranslationButton_clicked'
        QtMocHelpers::SlotData<void()>(50, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_delayedTranslateScreenAreaButton_clicked'
        QtMocHelpers::SlotData<void()>(51, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'clearText'
        QtMocHelpers::MethodData<void()>(52, 4, QMC::AccessPrivate | QMC::MethodScriptable, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->translationRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Language>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Language>>(_a[3]))); break;
        case 1: _t->translationAccepted(); break;
        case 2: _t->resetTranslator(); break;
        case 3: _t->ttsStateChanged((*reinterpret_cast<std::add_pointer_t<QTextToSpeech::State>>(_a[1]))); break;
        case 4: _t->onTTSError((*reinterpret_cast<std::add_pointer_t<QTextToSpeech::ErrorReason>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->translatorStateChanged((*reinterpret_cast<std::add_pointer_t<ATranslationProvider::State>>(_a[1]))); break;
        case 6: _t->open(); break;
        case 7: _t->quit(); break;
        case 8: _t->openSettings(); break;
        case 9: _t->toggleOcrNegate(); break;
        case 10: _t->on_translateButton_clicked(); break;
        case 11: _t->translateSelection(); break;
        case 12: _t->speakSelection(); break;
        case 13: _t->speakTranslatedSelection(); break;
        case 14: _t->stopSpeaking(); break;
        case 15: _t->playPauseSpeaking(); break;
        case 16: _t->copyTranslatedSelection(); break;
        case 17: _t->recognizeScreenArea(); break;
        case 18: _t->translateScreenArea(); break;
        case 19: _t->delayedRecognizeScreenArea(); break;
        case 20: _t->delayedTranslateScreenArea(); break;
        case 21: _t->setOrientation((*reinterpret_cast<std::add_pointer_t<Qt::ScreenOrientation>>(_a[1]))); break;
        case 22: _t->onSourceLanguageChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->onDestinationLanguageChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->handleAutoTranslation(); break;
        case 25: _t->updateVoiceComboBoxes(); break;
        case 26: _t->updateSpeakerComboBoxes(); break;
        case 27: _t->on_swapButton_clicked(); break;
        case 28: _t->on_abortButton_clicked(); break;
        case 29: _t->on_copySourceButton_clicked(); break;
        case 30: _t->on_delayedRecognizeScreenAreaButton_clicked(); break;
        case 31: _t->on_clearButton_clicked(); break;
        case 32: _t->on_copyTranslationButton_clicked(); break;
        case 33: _t->on_copyAllTranslationButton_clicked(); break;
        case 34: _t->on_delayedTranslateScreenAreaButton_clicked(); break;
        case 35: _t->clearText(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Language >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(const QString & , const Language & , const Language & )>(_a, &MainWindow::translationRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::translationAccepted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::resetTranslator, 2))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::translationRequested(const QString & _t1, const Language & _t2, const Language & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void MainWindow::translationAccepted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::resetTranslator()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
