/****************************************************************************
** Meta object code from reading C++ file 'appsettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../crow-translate/src/settings/appsettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11AppSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto AppSettings::qt_create_metaobjectdata<qt_meta_tag_ZN11AppSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppSettings",
        "LanguageFormat",
        "FullName",
        "IsoCode",
        "LanguageButtonsType",
        "Source",
        "Translation",
        "WindowMode",
        "PopupWindow",
        "MainWindow",
        "Notification",
        "RegionRememberType",
        "NeverRemember",
        "RememberLast",
        "RememberAlways",
        "IconType",
        "DefaultIcon",
        "LightIcon",
        "DarkIcon",
        "CustomIcon"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LanguageFormat'
        QtMocHelpers::EnumData<enum LanguageFormat>(1, 1, QMC::EnumFlags{}).add({
            {    2, LanguageFormat::FullName },
            {    3, LanguageFormat::IsoCode },
        }),
        // enum 'LanguageButtonsType'
        QtMocHelpers::EnumData<enum LanguageButtonsType>(4, 4, QMC::EnumFlags{}).add({
            {    5, LanguageButtonsType::Source },
            {    6, LanguageButtonsType::Translation },
        }),
        // enum 'WindowMode'
        QtMocHelpers::EnumData<enum WindowMode>(7, 7, QMC::EnumFlags{}).add({
            {    8, WindowMode::PopupWindow },
            {    9, WindowMode::MainWindow },
            {   10, WindowMode::Notification },
        }),
        // enum 'RegionRememberType'
        QtMocHelpers::EnumData<enum RegionRememberType>(11, 11, QMC::EnumFlags{}).add({
            {   12, RegionRememberType::NeverRemember },
            {   13, RegionRememberType::RememberLast },
            {   14, RegionRememberType::RememberAlways },
        }),
        // enum 'IconType'
        QtMocHelpers::EnumData<enum IconType>(15, 15, QMC::EnumFlags{}).add({
            {   16, IconType::DefaultIcon },
            {   17, IconType::LightIcon },
            {   18, IconType::DarkIcon },
            {   19, IconType::CustomIcon },
        }),
    };
    return QtMocHelpers::metaObjectData<AppSettings, qt_meta_tag_ZN11AppSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AppSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AppSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11AppSettingsE_t>.metaTypes,
    nullptr
} };

void AppSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppSettings *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *AppSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AppSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AppSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AppSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
